@@ -388,6 +388,8 @@ static void add_bookmarks(DBHashTable * dbh)
           bm_xfdir.gl[bm_xfdir.pathc].en=mk_net_entry(fullpath,target_type);
     } else return;
     if(!bm_xfdir.gl[bm_xfdir.pathc].en) assert_not_reached();
+    
+    SET_TOP_BOOKMARK(bm_xfdir.gl[bm_xfdir.pathc].en->subtype);
          
     bm_xfdir.gl[bm_xfdir.pathc].pathv = g_strdup(p);
     bm_xfdir.pathc++;
@@ -405,6 +407,7 @@ int open_bookmarks (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * tr
 
     gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
     if (!en) return -1;   
+    prune_row(treemodel,iter,NULL,en);
     target_iter = iter;
     target_treeview = treeview;
     target_type = en->type;
