@@ -43,6 +43,7 @@
 #include "types.h"
 
 #include "modules.h"
+#include "smb_module.h"
 
 #include "menu.h"
 #include "menu_callbacks.h"
@@ -90,7 +91,7 @@ G_MODULE_EXPORT
 preferences_t preferences_titles[]={
 #ifdef USE_EXTENDPREFS
 	    {N_("Autoscroll"),"AUTOSCROLL",AUTOSCROLL,GDK_A},
-	    {N_("Text headers"),"TEXT_HEADERS",TEXT_HEADERS,GDK_B},
+	    /*{N_("Text headers"),"TEXT_HEADERS",TEXT_HEADERS,GDK_B},*/
 #endif
 	    {N_("Copy on drag"),"DRAG_DO_COPY",DRAG_DO_COPY,GDK_C},
 #ifdef USE_EXTENDPREFS
@@ -166,8 +167,8 @@ autotype_t autotype []= {
 	{".tar.bz2","tar --use-compress-program bunzip2 -xf",N_("Extract files from %s"),N_("Extract into"),1},
 	{".bz2","bunzip2",N_("uncompress %s"),NULL,1},
 	{".Z","uncompress",N_("uncompress %s"),NULL,1},
-	{".ZIP","unzip",N_("uncompress s"),NULL,1},
-	{".zip","unzip",N_("uncompress %s"),NULL,1},
+	{".ZIP","unzip",N_("uncompress %s"),N_("Extract into"),1},
+	{".zip","unzip",N_("uncompress %s"),N_("Extract into"),1},
 	
 	{".tgz","tar -xzf ",N_("Extract files from %s"),N_("Extract into"),1},
 	{".tgz","tar -tzf ",N_("Package contents %s"),NULL,1}, /* XXX remove when tarball expansion enabled */
@@ -314,7 +315,6 @@ static gchar *path_selection[]={
 
 static gchar *anything_selected_list[]={
     "sort1",
-    "go1",
     "descending1",
     "ascending1",
     "unsorted6",
@@ -325,11 +325,8 @@ static gchar *anything_selected_list[]={
     "group6",
     "mode6",
     
-    "sb_go_home_menuitem",
-    "sb_goto_jump_menuitem",
-    "sb_go_backward_menuitem",
-    "sb_go_forward_menuitem",
-    "sb_go_up_menuitem",
+    /* the go stuff is taken care of, mostly, at widgets.c, along
+     * with the sidebar...*/
     NULL
 };
 
@@ -634,28 +631,75 @@ set_auto_command(			int j,
 	      
 
 	      if (queued && *queued && output_arg) command=g_strdup(app);
-	      else command=g_strdup(MIME_mk_command_line(app,path,FALSE,FALSE));
-	      
-	      /* check for duplicates */
-	      for (jj=0;jj<j;jj++){
-		gchar *set_command=g_object_get_data(G_OBJECT(WIDGET(auto_C_name[jj])), "command");
-#ifdef DEBUG
-		printf("j=%d: %s == %s\n",jj,set_command,command);
-#endif
-		if (strcmp(set_command,command)==0)
-		{ /* skip it */
-		  g_free(command);
-		  return j;
-		}	   
+	      else {
+		  if (strncmp(path,"//",strlen("//"))==0){
+		      gchar *netpath=g_build_filename(XF_SMBget_netfile_cache_dir(),strrchr(path,'/')+1,NULL);
+		      command=g_strdup(MIME_mk_command_line(app,netpath,FALSE,FALSE));
+		      g_free(netpath);
+		  }
+		  else command=g_strdup(MIME_mk_command_line(app,path,FALSE,FALSE));
 	      }
 	      
-	      if ((old_text=g_object_get_data(G_OBJECT(a),"command")) != NULL) g_free(old_text);
-	      if ((old_text=g_object_get_data(G_OBJECT(a),"output_arg")) != NULL) g_free(old_text);
-	      g_object_set_data(G_OBJECT(a),"command",(gpointer)command);
+	      /* check for duplicates */
+	      {
+		gint argcp1;
+		gchar **argvp1;
+		GError *error=NULL;
+		 if (!g_shell_parse_argv ((const gchar *)command,&argcp1,&argvp1,&error)){
+		    g_free(command);
+		    g_error_free(error);
+		    g_strfreev(argvp1);
+		    return j;
+		 }
+	         for (jj=0;jj<j;jj++){
+		   gboolean is_duplicate=TRUE;
+		   gint ac,argcp2;
+		   gchar **argvp2;
+		   gchar *set_command=g_object_get_data(G_OBJECT(WIDGET(auto_C_name[jj])), "command");
+		   TRACE("j=%d: %s == %s\n",jj,set_command,command);
+		   if (!g_shell_parse_argv ((const gchar *)set_command,&argcp2,&argvp2,&error)){
+		    g_free(command);
+		    g_error_free(error);
+		    g_strfreev(argvp2);
+		    return j;
+		   }
+		   if (argcp2 != argcp1) {
+		    is_duplicate=FALSE;
+		   } else for (ac=0; ac<argcp2; ac++){
+		    if (strcmp(argvp2[ac],argvp1[ac])){
+			is_duplicate=FALSE;
+			break;
+		    }
+		   }
+		   TRACE("0x%x, 0x%x\n",argvp1,argvp2);
+		
+		   g_strfreev(argvp2);
+		   argvp2=NULL;
+		   /*if (strcmp(set_command,command)==0)*/
+		   if (is_duplicate) {
+		      g_free(command);
+		      return j;
+		   }
+		 } /* end for */
+		 g_strfreev(argvp1);
+		 argvp1=NULL;
+	      }
 	      
+	      if ((old_text=g_object_get_data(G_OBJECT(a),"command")) != NULL) {
+		  g_object_set_data(G_OBJECT(a),"command",NULL);
+		  g_free(old_text);
+	      }
+	      if ((old_text=g_object_get_data(G_OBJECT(a),"output_arg")) != NULL) {
+		  g_object_set_data(G_OBJECT(a),"output_arg",NULL);
+		  g_free(old_text);
+	      }
+	      g_object_set_data(G_OBJECT(a),"command",(gpointer)command);
 	      g_object_set_data(G_OBJECT(a),"querypath",(gpointer)querypath);
 	      g_object_set_data(G_OBJECT(a),"queued",(gpointer)queued);
-	      g_object_set_data(G_OBJECT(a),"output_arg",((output_arg)?g_strdup(output_arg):NULL));
+	      if (output_arg) {
+		  g_object_set_data(G_OBJECT(a),"output_arg",g_strdup(output_arg));
+	      }
+	      TRACE("command=%s, output_arg=%s",command,output_arg);
 	      g_object_set_data(G_OBJECT(a),"output_ext",(gpointer)output_ext);
 	      showit(popup, name);
 	      j++;
@@ -938,6 +982,7 @@ void do_select_popup (GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
         showit(popup, "show_hidden1");
         showit(popup, "paste1_menuitem"); 
 	showit(popup, "popup_separator1");
+	if (IS_NETFILE(en->subtype)) autostuff(popup,en);
 	if (IS_XF_NETSHARE(en->subtype)){
 	      showit(popup, "mountP");
 	}
